;+=----------------------------------------------------------------------------
; Name   : example_04.pro
; Desc   : Example 4, covering the event mechanism in mrdisplay
;          The event mechanism in mrmultidisplay works in an analogous fashion.
; Author : (c) Oliver Thilmann, Aug 2003
;=-----------------------------------------------------------------------------
pro example_04

	mrlib_init

		; The mrdisplay object intercepts mouse events and creates additional
		; events if the image data (via setImage or updateImage) or the
		; palette (via setPalette) has been changed.

		; This program demonstrates how to make use of events.
		; Also have a look at the code of mrscroll, which is pretty simple
		; and demonstrates how to dispatch mouse events.

		; Use global vars for simplicity

	common globals4, display, contrast, dataflash, dcm

		; First create a small UI, consisting of the mrdisplay object and
		; a load button, as we want to get imageChange events
		; We also add the contrast and dataflash plugins - dataflash is
		; now attached to mouse button 1.

	mrdicom_setUIDs, 'mrimg', '0.0.00.0.0000.0.0'

	base     = widget_base( /column )
	display  = obj_new('mrdisplay', base, 512, 512, 512, 512, interpolate=0 )
	contrast = obj_new('mrcontrast', base, xsize = 512, ysize = 100 , colbars=2)
	button   = widget_button( base, value='Load image', /align_center, event_pro='loadbuttonpress')

	dataflash = obj_new('mrdataflash')

	widget_control, base, /realize

		; register the plugins

	contrast->selfregister,  display, button=3
	dataflash->selfregister, display, button=1

	display->activate

	;----------------------------------------------------------------
	; Event handling
	;----------------------------------------------------------------

		; now for the interesting part: create an imageChangeHandler
		; object as defined below

	evtobj = obj_new('imageChangeHandler')

		; register this object with the display object for image change and
		; palette change events (the latter one is not used any further)
		; Event names are case insensitive

	display->registerEvt, evtobj, ['ImgChange', 'PaletteChange']

		; display will from now on call the mrdisplayEvtHandler method from
		; the imageChangeHandler object whenever one of setImage, updateImage
		; or setPalette methods is invoked

		; IMPORTANT:
		; The callback objects are called in the order in which they registered!

		; To unregister, say from PaletteChange events, call
		; display->unregisterEvt, evtobj, ['PaletteChange']
		; However it is more common to unregister the whole object via
		; display->unregisterObj, evtobj


		; As for the current program, we still need the xmanager

	xmanager, 'example_04', base, event_handler='event_handler', /no_block

end

; ---------------------------------------------------------------------------
; Name: loadbuttonpress
; Desc: Loading a new image will present a file selector box to the user and,
;       upon selection of a valid file, load this file and display the image
;       and the dicom data.
; ---------------------------------------------------------------------------
pro loadbuttonpress, evt
	common globals4, display, contrast, dataflash, dcm

		; destroy possible old mrdata object

	if( obj_valid(dcm) ) then obj_destroy, dcm

	dcm  = obj_new('mrdicom')
	file = dialog_pickfile()
	res = dcm->read( file )
	if( res eq 0 ) then begin
		x = dialog_message(['Error loading image', dcm->errMsg()])
		obj_destroy, dcm
		dataflash->hide          ; nothing to display
		return
	end

		; set the image and display it
		; notice how the contrast bars are automatically updated - the contrast
		; object receives imageChange events and processes them.

	img = rotate( *(dcm->img()), 7 )
	display->setImg, ptr_new( img, /no_copy )
	display->fitToScreen
	display->display

		; Set the new dicom source for dataflash
		; If the dataflash object (i.e. data) is currently displayed, this will
		; automatically redisplay and thus update the displayed data.

	dataflash->setDicomSource, dcm

end

; dummy handler does nothing
pro event_handler, evt
end


; ---------------------------------------------------------------------------
; Name: imageChangeHandler
; Desc: The object which registers for image change events with the mrdisplay
;       object. To receive the events, the object has to provide a
;       mrdisplayEvtHandler method with one argument
; ---------------------------------------------------------------------------

pro imageChangeHandler::mrdisplayEvtHandler, evt

	x = dialog_message(['New image loaded','Some event structure values printed to stdout'], /info)
	print, 'Event type: ', (*evt).type    ; '5' is image change

		; get the display object which caused the event:

	display = (*evt).display
	print, 'Image x size       : ', display->imgXsize()
	print, 'Image y size       : ', display->imgYsize()
	print, 'Image minimum value: ', display->imgMinval()
	print, 'Image maximum value: ', display->imgMaxval()

end


; constructor and object definition


function imageChangeHandler::init
	return, 1
end

pro imageChangeHandler__define
	obj = { IMAGECHANGEHANDLER, i: 0 }
end


