;+=----------------------------------------------------------------------------
; Name   : example_07.pro
; Desc   : Example 7: mrroi callback functionality
;          We use example_06 as a basis for this program. This time however,
;          roi evaluation shall be done automatically whenever the user adds or
;          changes a roi.
;          This can be accomplished via mrRoi's notification mechanism where
;          mrRoi knows two event types: 'modify' and 'change'.
;          'modify' notifications are created whenever the user is about to
;          modify or add a roi - i.e. *while* a roi is modified or created.
;
;          'change' notifications are created the moment the user has finished
;          creating or modifying a roi (i.e. has released the mouse button)
;          'change' notifications are also send after rois have been loaded or
;          deleted.
;
;          'modify' notifications are meant for quick feedback to the user. As
;          they are numerous, the corresponding callback routine should process fast.
;          'change' notification handling might take as much time as needed.
;
;          Since callback only works on objects, we need to define an object
;          which implements a handler method. See below.
;
; Author : (c) Oliver Thilmann, June 2004
;=-----------------------------------------------------------------------------
pro example_07

	mrlib_init

		; commons even work with objects :) (urgh)

	common globals7, roiobj, text, img

		; load image

		; load image

	mrdicom_setUIDs, 'mrimg', '0.0.00.0.0000.0.0'
	file = dialog_pickfile( path='c:\' )

	dcm  = obj_new('mrImageSetDicom', 1)
	res  = dcm->load( file, 0, /rotate )

		; in case of errors, print error message and return

	if( res eq 0 ) then begin
		c = dialog_message(['Error loading file', dcm->errMsg()])
		return
	end

	img = (*(dcm->imgData()))[0]

		; CREATE UI, consisting of an mrdisplay object with contrast
		; functionality, the mrroi object and a text object for
		; result display

	base    = widget_base( /row )

	display = obj_new('mrdisplay', base, 512, 512, 512, 512, interpolate=0 )

		; this time no display of greyscale bars for the contrast object

	contrast = obj_new('mrcontrast')
	zoom     = obj_new('mrzoom')

		; create roi object incl. load/save buttons and display style buttons

	roiobj  = obj_new('mrroiui', base, /column, spacing=10, /frame )
	text    = widget_text( base, scr_ysize=250, /frame, /scroll, font='fixedsys' )

		; realize the UI

	widget_control, base, /realize

		; Register the plugins with the mrdisplay object and set the image

	contrast->selfregister, display, button=3
	zoom->selfregister, display, button=2
	display->setImg, img

		; scale image, so it fits the screen, display the image and activate events
		; the display must be activated, or the roi object will not receive any
		; events and thus be disfunctional.

	display->fitToScreen
	display->display
	display->activate

		; register the roi event with the mrdisplay object, so it receives
		; mouse events from the display.
		; The roi object must not be registered before a valid image is set
		; in the display object.
		; (imgchange events only came later - will be cured in the future)

	roiobj->selfregister, display

		; activate roi buttons

	roiobj->activate

		; so far everything as in example_06.pro
		; no instanciate the roievt object

	roievthandler = obj_new('roievt')

		; register the roievthandler with the roiobj

	roiobj->notify, roievthandler, 'mrroiEvtHandler', ['modify','change']

		; finally start the xmanager

	xmanager, 'example_06', base, event_handler='dummy_handler', /no_block

end

; dummy handler does nothing - just intercepts events
pro dummy_handler, evt
end


; ---------------------------------------------------------------------------
; Name: roievt__define
; Desc: the object to register with mrroi
;       held very simple - the event handler calls the static function for
;       roi evaluation
; ---------------------------------------------------------------------------
function roievt::init
	return, 1
end

pro roievt__define
	object = { ROIEVT, $
				i : 0 }
end

pro roievt::mrroiEvtHandler, sender, type, evt
		; the event type is '1' for roiupdate and '0' for roichange events
	evaluate_rois, (*evt).type
end


; ---------------------------------------------------------------------------
; Name: evaluate_rois, event_type
; Desc: Evaluate rois as in example_06.pro
;       Depending on the event_type as handed over from the roievt object,
;       add a '(done)' (for change) or '(active)' (for modify) to
;       the text window.
;       Newly created rois have number '999' while being defined.
; ---------------------------------------------------------------------------
pro evaluate_rois, event_type

	common globals7, roiobj, text, img

	; rois are stored in an IDLanRoiGroup object. Get that object
	; from our roi object and count.

	roigroup = roiobj->roiGroup()
	nrois    = roigroup->count()

	; start new text in textwidget

	widget_control, text, set_value='  ROI EVALUATION'
	widget_control, text, set_value='------------------', /append
	if( event_type eq 2 ) then begin
		widget_control, text, set_value='(done)', /append
	end else begin
		widget_control, text, set_value='(active)', /append
	end

	; if no rois defined we're already done

	if( nrois eq 0 ) then return

	; otherwise loop over all rois, calculate values and display

	for i = 0, nrois-1 do begin

			; get the roi from the goigroup container
		roi = roigroup->get( position=i )

			; the position does not necessarily correspond to the roinumber
			; get that from the roi itself (roi objects extend IDLanRoi)

		roino = roi->id()

			; get the mask of the roi. getFramedMask would provide only
			; the outline - here we're interested in the full roi
			; npixels will hold the number of pixels in the roi
			; In the process of roi creation, it may happen that npixels equals
			; '0' - in this case we continue with the next roi
		mask  = where( *(roi->filledMask()) gt 0, npixels)
		if( npixels eq 0 ) then continue

			; calculate the mean value of image points in the roi
		meanval = mean( (*img)[mask] )

			; and display the result in the text widget
		widget_control, text, set_value=' ', /append
		if( roino eq 999 ) then begin
			widget_control, text, set_value='Roi      : active', /append
		end else begin
			widget_control, text, set_value=string( roino,   format='("Roi      :",i5)'), /append
		end
		widget_control, text, set_value=string( npixels, format='("# Pixels :",i5)'), /append
		widget_control, text, set_value=string( meanval, format='("Mean     :",f7.1)'), /append

	end

end
