;+=--------------------------------------------------------------------------------------------
; Name  : createTrueColorImage
; RCS   : $Id: createTrueColorImg.pro,v 2.0 2004/08/05 17:56:30 olivert Exp olivert $
; Syntax: image = createTrueColorImage( sourceImage, red, green, blue, /flip, palette=palette )
; Args  : <b>sourceImage</b>
;         Pointer to the image matrix to convert from greyscale to a true color image
;         <b>red green blue</b>
;         Three arrays of type int[256] with the red, green and blue values
;         <b>palette</b>
;         Instead of the single red/green/blue arrays an IDLgrPalette object can be passed.
;         If a palette object is given, parameters red/green/blue are ignored!
;         <b>flip</b>
;         Name of the grasshopper in 'Maja, the bee'
;         Alternatively a keyword which, when set, will cause the resulting image
;         to be rotated via rotate, 7
; Ret   : a pointer to the newly allocated [3,*,*] result image
; Desc  : Create a true color image from an indexed image and a color palette
;         In case of errors the function returns a NULL pointer.
;         The function allocates new memory for the result image.
;         Known bugs: should provide some descriptive messages in case of errrors
; Author: (c) Jimmi Laett (Jimmy.Latt@radfys.lu.se)
;         7/2004: restructured parts, added palette option (O.T.)
; Licns : This program is published under the GPL. See gpl.txt for details
; Code  :
; >  img   = indgen( 256, 256 )
; >  red   = ingden( 256 )
; >  green = indgen( 256 )
; >  blue  = indgen( 256 )
; >  tcimg = createTrueColorImg( ptr_new(img), red, green, blue, /flip )
;=--------------------------------------------------------------------------------------------
function createTrueColorImg, img, red, green, blue, palette=palette, flip=flip

	if( n_elements(img) eq 0 ) then return, ptr_new()
	if( keyword_set(flip)    ) then src = rotate(*img,7) else src=*img

	r = ( n_elements(red)   ne 0 ) ? red   : 0
	g = ( n_elements(green) ne 0 ) ? green : 0
	b = ( n_elements(blue)  ne 0 ) ? blue  : 0

	if( keyword_set(palette) ) then begin
		if( not obj_valid( palette ) ) then return, ptr_new()
		palette->getProperty, red_values=r, green_values=g, blue_values=b
	end

	maxval = max( src )
	if( n_elements(r) lt maxval or n_elements(g) lt maxval or n_elements(b) lt maxval ) then return, ptr_new()

	imgSize  = size(src, /dimensions)
	tcimg    = intarr(3,imgSize[0], imgSize[1] )
	For x = 0, imgSize[0]-1 do begin
		for y = 0 , imgSize[1]-1 do begin
			index = src[x,y]
			tcimg[0,x,y] = R[index]
			tcimg[1,x,y] = G[index]
			tcimg[2,x,y] = B[index]
		end
	end
	tcimg = ptr_new( tcimg, /no_copy )
	return, tcimg
end
