;+=----------------------------------------------------------------------------
; Name   : mrdataflash
; RCS    : $Id: mrdataflash__define.pro,v 2.1 2004/11/10 10:44:04 olivert Exp olivert $
; Methods:
;          <b>Constructor</b>
;          mrdataflash::init                : constructor
;          <br>
;          <b>General</b>
;          mrdataflash::display             : display data
;          mrdataflash::hide                : switch off data display
;          mrdataflash::show                : same as mrdataflash::display
;          <br>
;          <b>Get and set</b>
;          mrdataflash::setDicomSource      : set the mrdicom data source object
;          mrdataflash::setCharsize         : set character size
;          <br>
;          <b>Event handling</b>
;          mrdataflash::selfregister        : register with a display object
;          mrdataflash::unregister          : unregister with the display
;          mrdataflash::mrdisplayEvtHandler : event handler for the mrdisplay
;          mrdataflash::buttonPressEvt      : toggles data display
;          <br>
;          <b>Private</b>
;          mrdataflash__define              : constructor, part I
;          mrdataflash::cleanup             : destructor
; Desc   : A plugin object for mrdisplay and mrmultidisplay which overlays
;          patient data to the image.
;          The first call to display will be in many cases a bit slow as
;          the underlying mrdicom object needs to fill its internal table
;          of tagvalues (which can take up to 1-2 seconds on a PIII 700!)
;          <br>
; Code   :
;       display = obj_new('mrdisplay')
;                                         ; now set some image, etc
;       flash   = obj_new('mrdataflash')
;       flash->selfregister, display
;       flash->setDicomSource, some_mrdicom_object
;       flash->display                     ; display data, will call display->display
;                                          ; then .... do whatever
;       flash->hide                        ; eventually hide date again
;
;       One may also link a mouse button to displaying/hiding the data, a la:
;       flash->selfregister, display, button=2
;       Pressing the mouse button will then show/hide the dicom tag values
;
; Author: Oliver Thilmann, oliver.thilmann@web.de, May 2003
; Licns : This program is published under the LGPL. See lgpl.txt for details
;=-----------------------------------------------------------------------------

;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrdataflash::init
; Syntax: obj = obj_new('mrDataflash', fontsize=float, color=intarr(3))
; Args  : <b>fontsize</b>
;         font size, default is 11 point
;         <b>color</b>
;         RGB font color, default is [0,255,0] (green)
; Ret   : 0 or 1 (i.e. null pointer or object pointer)
; Desc  : Instanciates the object
;=-----------------------------------------------------------------------------
function mrdataflash::init, fontsize=fontsize, color=color
	self.color    = n_elements(color)    eq 3 ? color    : [0,255,0]
	fontsize      = n_elements(fontsize) eq 1 ? fontsize : 11
	self.dicomSrc = obj_new()
	self.onoff    = 0              ; off
	self.font     = obj_new('IDLgrFont', size=fontsize )
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::cleanup
; Desc  : remove data (hide) from display, then unregister
;=-----------------------------------------------------------------------------
function mrdataflash::cleanup
	self->hide
	self->unregister
end


;+=----------------------------------------------------------------------------
; Name  : mrdataflash::setDicomSource
; Syntax: obj->setDicomSource, sourceobj
; Args  : <b>sourceobj</b>
;         A object reference to an mrdicom object to use as data source
; Desc  : Informs the mrdataflash from which mrdicom object to read the data
;         If data is displayed, calling this function will automatically
;         redisplay
;=-----------------------------------------------------------------------------
pro mrdataflash::setDicomSource, source
	if( not obj_valid(source) ) then return
	if( not obj_isa(source, 'mrdicom') ) then message, 'source must be of type mrdicom'
	self.dicomSrc = source
	if( self.onoff eq 1 ) then self->display
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::setFontsize
; Syntax: obj->setFontsize, fontsize
; Args  : <b>fontsize</b>
;         The font size in pixels.
; Desc  : If mrdataflash values are visible, changing the character size
;         will automatically redisplay. Setting the font size too large in
;         a small window will make the text overlap.
;=-----------------------------------------------------------------------------
pro mrdataflash::setFontsize, size
	if( n_elements( size ) eq 0 ) then return
	self.font->setProperty, size=size
	if( self.onoff ) then self->display
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::setColor
; Syntax: obj->setColor, color
; Args  : <b>color</b>
;         3-element integer array holding the RGB colors to use.
;         Setting the color will automatically redisplay
; Desc  : Sets the color. If color is not a 3-element vector, the method returns
;         without further ado.
;=-----------------------------------------------------------------------------
pro mrdataflash::setColor, color
	if( n_elements(color) ne 3 ) then return
	self.color = color
	if( self.onoff ) then self->display
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::display
; Syntax: obj->display
; Desc  : Display the data - each call to mrdataflash::display will re-read
;         the data from the mrdicom object!
;         (Local storage and re-read upon request puts more load on the user
;         without any real benefit as getting the data from the mrdicom
;         object is not expensive - so we re-read. Otoh that means, that the
;         mrdicom object must not be destroyed in the meantime....
;=-----------------------------------------------------------------------------
pro mrdataflash::display
	if( not obj_valid( self.display ) ) then return ; dunno where to display
	if( self.dicomSrc eq obj_new() ) then return    ; nothing to display
	if( not obj_valid( self.dicomSrc ) ) then begin ; mrdicom obj was destroyed
		x = dialog_message('Dicom object destroyed in mrdataflash::display', /error)
		return
	end

		; set the graphics 'environment'

	view = self.display->labelView()
	if( obj_valid( self.textModel ) ) then begin
		view->remove, self.textModel
		obj_destroy, self.textModel                 ; destroys all submodels, too
	end
	self.textModel = obj_new('IDLgrModel')

		; now add the text objects to our model

	text = self.dicomSrc->getTagValue('pat_patient_name')
	txt = obj_new('IDLgrText', text, /onglass, locations=[.01,.99], alignment=0.0,vertical_alignment=1.0, color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('pat_patient_id')
	txt = obj_new('IDLgrText', text, locations=[.01,.96], alignment=0.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	date = self.dicomSrc->getTagValue('pat_patient_birthdate')
	text = '*'+strmid(date,6,2)+'/'+strmid(date,4,2)+'/'+strmid(date,0,4)+';'+self.dicomSrc->getTagValue('pat_patient_age')
	txt = obj_new('IDLgrText', text, locations=[.01,.93], alignment=0.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('pat_medical_alerts')
	txt = obj_new('IDLgrText', text, locations=[.01,.90], alignment=0.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	date = self.dicomSrc->getTagValue('id_image_date')
	text = strmid(date,6,2)+'/'+strmid(date,4,2)+'/'+strmid(date,0,4)
	txt = obj_new('IDLgrText', text, locations=[.01,.87], alignment=0.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt


	text = self.dicomSrc->getTagValue('id_series_description')
	txt = obj_new('IDLgrText', text, locations=[.01,.84], alignment=0.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = strtrim( string(self.dicomSrc->getTagValue('rel_series_number')),2) + ' IMA ' + $
	       strtrim( string(self.dicomSrc->getTagValue('rel_image_number')),2)
	txt = obj_new('IDLgrText', text, locations=[.01,.81], alignment=0.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

		; upper right

	text = self.dicomSrc->getTagValue('id_manufacturer') + ' ' + self.dicomSrc->getTagValue('id_manufacturer_model_name')
	txt = obj_new('IDLgrText', text, locations=[.99,.99], alignment=1.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('meta_implementation_version_name')
	txt = obj_new('IDLgrText', text, locations=[.99,.96], alignment=1.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('id_institution_name')
	txt = obj_new('IDLgrText', text, locations=[.99,.93], alignment=1.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('id_study_description')
	txt = obj_new('IDLgrText', text, locations=[.99,.90], alignment=1.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('acq_patient_position')
	txt = obj_new('IDLgrText', text, locations=[.99,.87], alignment=1.0,vertical_alignment=1.0,color=self.color, font=self.font )
	self.textModel->add, txt

			; lower left

	ti = self.dicomSrc->getTagValue('acq_inversion_time')
	if( ti ne 0 ) then begin                              ; display only if inversion time != 0
		text = 'TI ' + strtrim( string(fix(ti)),2)
		txt = obj_new('IDLgrText', text, locations=[.01,.13], alignment=0.0,vertical_alignment=0.0,color=self.color, font=self.font )
		self.textModel->add, txt
	end

	text = 'TR ' + strtrim( string(fix(self.dicomSrc->getTagValue('acq_repetition_time'))),2)
	txt = obj_new('IDLgrText', text, locations=[.01,.10], alignment=0.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = 'TE ' + strtrim( string(fix(self.dicomSrc->getTagValue('acq_echo_time'))),2)
	txt = obj_new('IDLgrText', text, locations=[.01,.07], alignment=0.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = 'BW ' + strtrim( string(fix(self.dicomSrc->getTagValue('acq_pixel_bandwidth'))),2)
	txt = obj_new('IDLgrText', text, locations=[.01,.04], alignment=0.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('acq_sequence_name') + ' / ' + strtrim( string(fix(self.dicomSrc->getTagValue('acq_flip_angle'))),2)
	txt = obj_new('IDLgrText', text, locations=[.01,.01], alignment=0.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

		; lower right

	text = string(self.dicomSrc->getTagValue('acq_spacing_between_slices'), format='("SP ", f4.1)')
	txt = obj_new('IDLgrText', text, locations=[.99,.16], alignment=1.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = string(self.dicomSrc->getTagValue('acq_slice_thickness'), format='("SL ", f4.1)')
	txt = obj_new('IDLgrText', text, locations=[.99,.13], alignment=1.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = strtrim(string(self.dicomSrc->getTagValue('img_rows')),2) + ' * ' + $
           strtrim( string(self.dicomSrc->getTagValue('img_columns')),2)
	txt = obj_new('IDLgrText', text, locations=[.99,.10], alignment=1.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = 'PE ' + self.dicomSrc->getTagValue('acq_phase_encoding_direction')
	txt = obj_new('IDLgrText', text, locations=[.99,.07], alignment=1.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = self.dicomSrc->getTagValue('acq_protocol_name')
	txt = obj_new('IDLgrText', text, locations=[.99,.04], alignment=1.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

	text = 'LO ' + strtrim( string(self.dicomSrc->getTagValue('rel_slice_location')),2)
	txt = obj_new('IDLgrText', text, locations=[.99,.01], alignment=1.0,vertical_alignment=0.0,color=self.color, font=self.font )
	self.textModel->add, txt

		; and finally display what we got

	view->add, self.textModel
	self.display->display
	self.onoff = 1
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::hide
; Syntax: obj->hide
; Desc  : Removes dicom data from display. Causes redisplay.
;=-----------------------------------------------------------------------------
pro mrdataflash::hide
	if( not obj_valid( self.display ) ) then return
	if( not obj_valid( self.textModel ) ) then return
	view = self.display->labelView()
	view->remove, self.textModel
	obj_destroy, self.textModel
	self.textModel = obj_new()
	self.display->display
	self.onoff = 0
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::show
; Syntax: obj->show
; Desc  : Displays dicom data. Same as mrdataflash::display
;=-----------------------------------------------------------------------------
pro mrdataflash::show
	self->display
end


;==============================================================================
;
;                                  EVENT HANDLING
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrdataflash::selfregister
; Syntax: obj->selfregister, display, button=int
; Args  : <b>display</b>
;         The mrdisplay object with which to register
;         <b>button</b>
;         Integer denoting the mouse button which toggles display. Left = 1,
;         middle=2, right=3. By default, no (=0) mouse button is associated
;         with the object.
; Desc  : Register with the display
;=-----------------------------------------------------------------------------
pro mrdataflash::selfregister, display, button=button
	if( n_elements(button) eq 0 ) then button = 0
	if( button lt 0 ) then begin
		print, 'Illegal button number in mrdataflash::selfregister'
		return
	end
	display->registerEvt, self, ['Press']
	self.display = display
	self.button  = 2^(button-1)   ; integer arithmetic: 2^-1 = 0
	return
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::unregister
; Syntax: obj->unregister
; Desc  : Unregisters with a mrdisplay object
;=-----------------------------------------------------------------------------
pro mrdataflash::unregister
	if( not obj_valid(self.display) ) then return
	self.display->unregisterObj, self
	self.display = obj_new()
end


;+=----------------------------------------------------------------------------
; Name  : mrdataflash::mrdisplayEvtHandler
; Syntax: obj->mrdisplayEvtHandler
; Args  : <b>evt</b>
;         Pointer to the event structure from mrdisplay
; Desc  : Dispatch the event coming from mrdisplay
;=-----------------------------------------------------------------------------
pro mrdataflash::mrdisplayEvtHandler, evt
	if( self.button eq 0 ) then return                  ; no button registered
	if( ((*evt).button and self.button) ne self.button ) then return
	if( (*evt).type ne 0 ) then return
	self->buttonPressEvt, evt
end

;+=----------------------------------------------------------------------------
; Name  : mrdataflash::buttonPressEvt
; Syntax: obj->buttonPressEvt, evt
; Args  : <b>evt</b>
;         Pointer to the event structure from mrdisplay
; Desc  : Toggle display of dicom data via calls to mrdataflash::display
;         and mrdataflash::hide
;=-----------------------------------------------------------------------------
pro mrdataflash::buttonPressEvt, evt
	if( self.onoff eq 0 ) then self->display else self->hide
	; self.onoff is set anew in the display and hide method
end


;==============================================================================
;
;                             PRIVATE OBJECT METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrdataflash__define
; Desc  : The constructor
;=-----------------------------------------------------------------------------
pro mrdataflash__define
	obj = { MRDATAFLASH, $
			display		:	obj_new()	,$
			button		:	0			,$
			onoff       :   0           ,$ ; flags whether data is displayed or not
			dicomSrc    :   obj_new()   ,$ ; the mrdicom object from which to retrieve data
			font		:	obj_new()	,$ ; the font used for display
			color		:	intarr(3)	,$ ; character color
			textModel   :   obj_new()    $ ; idlgrModel containing the idlgrtext objects
		}
end
