;+=----------------------------------------------------------------------------
; Name  : mrDialogSelect
; RCS   : $Id: mrdialog_select.pro,v 2.1 2004/09/21 17:49:32 olivert Exp olivert $
; Syntax: button = mrdialog_select( message, buttons, title=title, group_leader=gl,
;         .        default=int, cancel=int,
;         .        bsize=bsize, /question, /warning, /error, /info )
; Ret   : In case of argument errors the function returns -1
; Args  : <b>message</b>
;         String or array of strings for messages with more than one line.
;         <b>buttons</b>
;         String or array of strings containing the button texts
;         <b>title</b>
;         Sets the window title
;         <b>group_leader</b>
;         This is a modal widget, so the group leader <b>must</b> be set
;         <b>default_button</b>
;         Index of the default button, starting with '0'
;         <b>cancel_button</b>
;         Index of the cancel button
;         <b>bsize</b>
;         Sizes of the single buttons in pixels. bsize can either be a single integer
;         in which case all buttons will be of the same size, or an array with the
;         the number of elements corresponding to the number of buttons.
;         <b>question</b>
;         Display the question bitmap
;         <b>warning</b>
;         Display the warning bitmap (default)
;         <b>error</b>
;         Display the error bitmap
;         <b>info</b>
;         Display the info bitmap
; Desc  : A kind of dialog_message box, but with an arbitrary number of buttons.
;         The return value is the index of the selected button, starting from
;         '0' for the left most one. I find this more appropriate than returning
;         the button value, esp. when it comes to internationalization.
; Author: (c) Oliver Thilmann, oliver.thilmann@web.de
;         following a posting by Ben Tupper on comp.lang.idl_pvwave
; Code  : button_number = mrdialog_select(['A message in','2 lines'], ['button1','button2'], /info )
; Licns  : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------
function mrdialog_select, message, buttons, title=title, group_leader=group_leader, $
							default_button=def_id, cancel_button=cnl_id, $
							bsize=bsize, $
							question=question, warning=warning, error=error, info=info, skull=skull

	on_error, 2

		; check arguments and load image bitmap

	if( n_elements(buttons)      eq 0 ) then message, 'Empty button field'
	if( n_elements(group_leader) eq 0 ) then message, 'Missing group leader widget id'
	if(	widget_info( group_leader, /valid_id) eq 0 ) then message, 'Invalid group leader widget id'
	title   = ( n_elements(title)   eq 0 ) ? 'Please select' : title
	def_id  = ( n_elements(def_id)  eq 0 ) ? -1 : def_id
	cnl_id  = ( n_elements(cnl_id)  eq 0 ) ? -1 : cnl_id
	case n_elements(bsize) of
	0: begin
		bsize = intarr( n_elements(buttons) )
		bsize[*] = -1
	end
	1: begin
		tmp   = bsize
		bsize = intarr( n_elements(buttons) )
		bsize[*] = tmp
	end
	else: if( n_elements(bsize) ne n_elements(buttons) ) then message, 'Wrong number of elements in argument bsize'
	end
	image = !DIR + '\resource\bitmaps\mrdialog_warning.bmp'
	if( keyword_set(info)    ) then image = !DIR + '\resource\bitmaps\mrdialog_info.bmp'
	if( keyword_set(question)) then image = !DIR + '\resource\bitmaps\mrdialog_question.bmp'
	if( keyword_set(error)   ) then image = !DIR + '\resource\bitmaps\mrdialog_error.bmp'
	if( keyword_set(skull)   ) then image = !DIR + '\resource\bitmaps\mrdialog_skull.bmp'  ; undocumented (8=X
	img = read_bmp( image, /rgb)

		; the widget layout is ugly, as always

	cnl_wid = 0L
	def_wid = 0L
	base    = widget_base( title=title, /column, /base_align_center, group_leader=group_leader, /modal)
	base1   = widget_base( base, column=3, /align_right )
	draw    = widget_draw( base1, scr_xsize=36, scr_ysize=36 )
	x       = widget_base( base1, scr_xsize=10)
	mess    = widget_base( base1, /column, /base_align_right, /align_center )

	for i = 0, n_elements(message)-1 do begin
		label = widget_label( mess, value=message[i], /align_left )
	end
	base2 = widget_base( base, /row )
	for i = 0, n_elements(buttons)-1 do begin
		if( bsize[i] eq -1 ) then begin
			bt = widget_button( base2, value=buttons[i], uvalue=i, event_pro='mrdialog_select_evt', frame=frame )
		end else begin
			bt = widget_button( base2, value=buttons[i], uvalue=i, event_pro='mrdialog_select_evt', frame=frame, xsize=bsize[i] )
		end
		if( cnl_id eq i ) then cnl_wid = bt
		if( def_id eq i ) then def_wid = bt
		if( i ne n_elements(buttons)-1 ) then x  = widget_label( base2, value=' ' )
	end

		;
		; tell system about default and cancel buttons
		;

	if( def_wid ne 0L ) then widget_control, base, default_button=def_wid
	if( cnl_wid ne 0L ) then widget_control, base, cancel_button=cnl_wid

		;
		; the selected button is stored in the uvalue of the top-widget.
		;

	res = -1
	selection = ptr_new( res, /no_copy )
	widget_control, base, set_uvalue=selection

		; is there a better way for beautification??

	geom1  = widget_info( base1, /geometry )
	geom2  = widget_info( base2, /geometry )
	xsize  = geom1.scr_xsize > geom2.scr_xsize
	widget_control, base1, scr_xsize=xsize
	widget_control, mess, scr_xsize=xsize-56

		; place widget in the middle of the screen

	geom   = widget_info( base1, /geometry )
	screen = get_screen_size()
	widget_control, base, xoffset=screen[0]/2-geom.scr_xsize/2-geom.margin, $
	                      yoffset=screen[1]/2-geom.scr_ysize/2-geom.margin

	widget_control, base, /realize
	tv, img, /true
	xmanager, 'mrdialog_select', base, event_handler='mrdialog_select_evt', group_leader=base

	button = *selection
	ptr_free, selection
	return, button
end

pro mrdialog_select_evt, evt
	widget_control,  evt.id, get_uvalue = value
	widget_control, evt.top, get_uvalue = select
	*select = value
	widget_control, evt.top, /destroy
end
