;+=----------------------------------------------------------------------------
; Name    : mrError
; RCS     : $Id: mrerror__define.pro,v 2.0 2004/08/05 18:05:38 olivert Exp olivert $
; Desc    : An object to provide logging and error handling functionality.
;           Logging is relatively straight forward. Once instanciated (where
;           it is defined whether output should go to a file or stdout), the
;           object provides log, warn, err and fatal methods which will create
;           a corresponding message line in the output file.
;           For warn and err messages, the user may set a certain limit after
;           which the program is terminated. fatal messages always lead to
;           immediate program termination.
;           The user may also register objects for callback and/or destruction
;           at the time of program termination. Objects registered for callback
;           must provide a procedure named 'fatal'.
;           Objects are called before program termination in reverse order in
;           which they were registered.
;           <br>
;           <b>Constructors and destructor</b>
;           mrError__define           : constructor, 1st part
;           mrError::init             : constructor, 2nd part
;           mrError::cleanup          : the destructor
;           <b>General</b>
;           mrError::log              : print informational message
;           mrError::warn             : print warning message
;           mrError::error            : print error message
;           mrError::fatal            : print fatal message and exit
;           mrError::add              : add object to callback and/or destroy list
;           mrError::remove           : remove object from lists
;           <b>Private methods</b>
;           mrError::mess             : called by all log methods
;           mrError::terminateProg    : handles callback at program termination
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------

;+=----------------------------------------------------------------------------
; Name  : mrError::init
; Syntax: obj = obj_new('mrerror', file, /append, warnlimit=w, errlimit=e )
; Args  : <b>file</b>
;         If present, output will go to the given filename, otherwise it
;         will be printed to stdout.
;         <b>append</b>
;         If set, append to a possibly existing file.
;         <b>warnlimit</b>
;         Maximum number of warning messages before program is terminated.
;         Default is '0', which means that warning messages never lead to
;         program termination.
;         <b>errlimit</b>
;         Maximum number of error messages before program is terminated.
; Desc  : Instanciates an mrError object. If a filename is present, but
;         opening fails, the object will not be created.
;=-----------------------------------------------------------------------------
function mrError::init, file, warnlimit=warnlimit, errorlimit=errlimit, append=append

		; handle arguments

	file   = n_elements(file)   eq 0 ? '' : file
	append = n_elements(append) eq 0 ?  0 : append

	self.warnlimit = n_elements(warnlimit) eq 0 ? 0 : warnlimit
	self.errlimit  = n_elements(errlimit)  eq 0 ? 0 : errlimit

		; some more default values

	self.lun       = 0
	self.ninfo     = 0
	self.nwarn     = 0
	self.nerr      = 0
	self.file      = file

		; if filename given, try to open a file

	if( file ne '' ) then begin
		openw, lun, file, /get_lun, append=append, error=err
		if( err ne 0 ) then begin
			print, 'Error instanciating mrError object'
			return, 0
		end
		self.lun = lun
		printf, self.lun, systime(0), ' Logging started'
	end
	self.Cfatal   = obj_new('IDL_Container')
	self.Cdestroy = obj_new('IDL_Container')
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mrError::cleanup
; Desc  : The destructor - clears and destroys the container objects
;=-----------------------------------------------------------------------------
pro mrError::cleanup
	self.Cfatal->remove, /all
	obj_destroy, self.Cfatal
	self.Cdestroy->remove, /all
	obj_destroy, self.Cdestroy
end

;+=----------------------------------------------------------------------------
; Name  : mrError::log
; Syntax: obj->log, message [, origin]
; Args  : <b>message</b>
;         The message to print
;         <b>origin</b>
;         Optional argument denoting the origin of the message. May be a
;         string or an object, in which case the class name is printed as
;         origin.
; Desc  : Print an informational message
;=-----------------------------------------------------------------------------
pro mrError::log, message, origin
	if( n_elements(origin) eq 0 ) then origin=''
	if( n_elements(message) eq 0 ) then message=''
	self->mess, 0, message, origin, 0
end

;+=----------------------------------------------------------------------------
; Name  : mrError::warn
; Syntax: obj->warn, message [, origin]
; Args  : see mrError::log
; Desc  : Print a warning message
;=-----------------------------------------------------------------------------
pro mrError::warn, message, origin
	if( n_elements(origin) eq 0 ) then origin=''
	self.nwarn = self.nwarn+1
	if( n_elements(message) eq 0 ) then message=''
	if( self.warnlimit gt 0 and self.nwarn ge self.warnlimit ) then exit=1 else exit=0
	self->mess, 1, message, origin, exit
end

;+=----------------------------------------------------------------------------
; Name  : mrError::error
; Syntax: obj->error, message [, origin]
; Args  : see mrError::log
; Desc  : Prints the given error message and a stackdump
;=-----------------------------------------------------------------------------
pro mrError::error, message, origin
	if( n_elements(origin) eq 0 ) then origin=''
	if( n_elements(message) eq 0 ) then message=''
	self.nerr = self.nerr+1
	if( self.errlimit gt 0 and self.nerr ge self.errlimit ) then exit=1 else exit=0
	self->mess, 2, message, origin, 0
end

;+=----------------------------------------------------------------------------
; Name  : mrError::fatal
; Syntax: obj->fatal, message [, origin]
; Args  : see mrError::log
; Desc  : Print 'FATAL' message and stacktrace and terminate program
;=-----------------------------------------------------------------------------
pro mrError::fatal, message, origin
	if( n_elements(origin) eq 0 ) then origin=''
	self->mess, 3, message, origin, 0
end

;+=----------------------------------------------------------------------------
; Name  : mrError::add
; Syntax: obj->add, obj, /fatal, /destroy
; Args  : <b>obj</b>
;         The object to add
;         <b>fatal</b> and <b>destroy</b>
;         Flags denoting the list to which to add the object
; Desc  : Adds an object to the 'fatal' and/or 'destroy' list.
;         At time of program termination, mrError calls the 'fatal' method
;         of all objects in the fatal list in reverse order in which they
;         were added. Afterwards, objects in the 'destroy' list are
;         explicitly destroyed, also in reverse order.
;=-----------------------------------------------------------------------------
pro mrError::add, obj, fatal=fatal, destroy=destroy
	fatal   = n_elements(fatal)   eq 0 ? 0 : fatal
	destroy = n_elements(destroy) eq 0 ? 0 : destroy
	if( n_elements(obj) eq 0 ) then return

	for i = 0, n_elements(obj) do begin
		if( not obj_valid(obj[i]) ) then continue
		if( fatal   ) then self.Cfatal->add, obj[i]
		if( destroy ) then self.Cdestroy->add, obj[i]
	end
end

;+=----------------------------------------------------------------------------
; Name  : mrError::remove
; Syntax: obj->remove, obj
; Args  : <b>obj</b>
;         The object to remove from the lists
; Desc  : Removes an object from both the 'fatal' and the 'destroy' lists.
;         If an object should have been registered more than once, all
;         entries will be removed.
;=-----------------------------------------------------------------------------
pro mrError::remove, obj
	if( n_elements(obj) eq 0 ) then return
	for i = 0, n_elements(obj) do begin
		self.Cfatal->remove, obj[i]
		self.Cdestroy->remove, obj[i]
	end
end

;+=----------------------------------------------------------------------------
; Name  : mrError::mess
; Syntax: self->mess, type, 'message', origin, /exit
; Args  : <b>type</b>
;         0 for informational message, 1 = warning, 2 = error, 3 = fatal
;         <b>message</b>
;         String containing the message to print
;         <b>origin</b>
;         Either a string or an object reference
;         <b>exit</b>
;         If set to '1', the program termination is started.
; Desc  : Private procedure!
;=-----------------------------------------------------------------------------
pro mrError::mess, type, mess, origin, exit

	if( obj_valid(origin) ) then txt = obj_class(origin) else txt=origin

	case type of
		0: message = strmid(systime(0),3,16) + ' INFO  ' + txt + ': ' + mess
		1: message = strmid(systime(0),3,16) + ' WARN  ' + txt + ': ' + mess
		2: message = strmid(systime(0),3,16) + ' ERROR ' + txt + ': ' + mess
		3: message = strmid(systime(0),3,16) + ' FATAL ' + txt + ': ' + mess
		4: message = '                            ' + mess
	end

	if( self.lun eq 0 ) then begin
		print, message
	end else begin
		printf, self.lun, message
	end

	if( exit ) then begin
		if( self.lun gt 0 ) then begin
			printf, self.lun, systime(0), ' Logging ended'
			free_lun, self.lun
		end
		self->terminateProg
	end

end

;+=----------------------------------------------------------------------------
; Name  : mrError::terminateProg
; Syntax: self->terminateProg
; Desc  : Call fatal methods of registered objects and destroy registered
;         objectecs before terminating the program
;=-----------------------------------------------------------------------------
pro mrError::terminateProg
	all = self.Cfatal->get( /all, count=n )
	for i = n-1, 0, -1 do all[i]->fatal
	all = self.CDestroy->get( /all, count=n )
	for i = n-1, 0, -1 do obj_destroy, all[i]
	exit, status=1    ; flag error
end

;+=----------------------------------------------------------------------------
; Name  : mrError__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mrerror__define
	obj = { MRERROR, $
			file		:		   ''	,$ ; filename, if we write to file
			lun			:			0	,$ ; logical unit associated with the filename
			ninfo		:			0	,$ ; number of info messages
			nwarn		:			0	,$ ; number of warning messages
			nerr		:			0	,$ ; number of error messages
			warnlimit	:			0	,$ ; maximum number of warning messages
			errlimit	:			0	,$ ; maximum number of error messages
			Cfatal		:	obj_new()	,$ ; an IDL_Container object
			Cdestroy	:	obj_new()	 $ ; another one
		}
end


