;+=----------------------------------------------------------------------------
; Name   : mrImageSetDicom
; RCS    : $Id: mrimagesetdicom__define.pro,v 2.3 2004/11/10 10:52:37 olivert Exp olivert $
; Supercl: mrImageSet
; Methods:
;         <b>Constructor</b>
;         mrImageSetDicom::init             : d1, d2, d3, name=name, class=class
;         <br>
;         <b>Public variables (get)</b>
;         mrImageSetDicom::name             : see mrImageSet::name
;         mrImageSetDicom::class            : see mrImageSet::class
;         mrImageSetDicom::xsize            : see mrImageSet::xsize
;         mrImageSetDicom::ysize            : see mrImageSet::ysize
;         mrImageSetDicom::d0               : see mrImageSet::d0
;         mrImageSetDicom::nz               : see mrImageSet::nz
;         mrImageSetDicom::nSlices          : see mrImageSet::nSlices
;         mrImageSetDicom::d1               : see mrImageSet::d1
;         mrImageSetDicom::nt               : see mrImageSet::nt
;         mrImageSetDicom::nShots           : see mrImageSet::nShots
;         mrImageSetDicom::d2               : see mrImageSet::d2
;         mrImageSetDicom::size             : see mrImageSet::size
;         mrImageSetDicom::nImgs            : see mrImageSet::nImgs
;         mrImageSetDicom::palette          : see mrImageSet::palette
;         mrImageSetDicom::imgdata          : see mrImageSet::imgdata
;         mrImageSetDicom::path             : see mrImageSet::path
;         mrImageSetDicom::seriesNumber     : series number of loaded images
;         mrImageSetDicom::studyUID         : unique study ID
;         mrImageSetDicom::dicomData        : returns pointer to mrDicom object matrix
;         <br>
;         <b>Public variables (set)</b>
;         mrImageSetDicom::setName          : see mrImageSet::setName
;         mrImageSetDicom::setClass         : see mrImageSet::setClass
;         <br>
;         <b>Image handling</b>
;         mrImageSetDicom::set              : see mrImageSet::set
;         mrImageSetDicom::get              : see mrImageSet::get
;         mrImageSetDicom::delete           : deletes one or all images from the set
;         mrImageSetDicom::condense         : see mrImageSet::condense
;         mrImageSetDicom::load             : load a single image
;         mrImageSetDicom::loadSequence     : load an image sequence
;         mrImageSetDicom::loadTimeSequence : load a 2-dim (time/space) image sequence
;         mrImageSetDicom::errNo            : see mrImageSet::errNo
;         mrImageSetDicom::errMsg           : see mrImageSet::errMsg
;         mrImageSetDicom::mrImageSet       : creates copy of the underlying mrImageSet
;         <br>
;         <b>Callback</b>
;         mrImageSetDicom::notify           : see mrImageSet::notify
;         <br>
;         <b>Private functions</b>
;         mrImageSetDicom::loadImgs         : the basic load method
;         mrImageSetDicom__define           : the constructor
;         mrImageSetDicom::cleanup          : the destructor
;         <br>
; Desc  : An image set of dicom images. Subclass to mrimageset with additional
;         means to store the DICOM objects and to load whole image sequences
;         at once.
;         A dataset is identified via its name and class - these two strings
;         are freely definable by the user - and the series number and the
;         unique study ID - these latter two come from the loaded dicom objects.
;
;         There's essentially two ways to use this object.
;
;         <ul>
;         <li> as a container for dicom objects which are loaded individually
;              one after the other via the load function
;         <li> as an object that holds a set of images loaded all at once and
;              automatically sorted into a matrix via the loadSequence method.
;         <li> as a container for user created dicom objects which will be
;              saved later.
;         </ul>
;
;         In the first case, the dimensions of the image matrix have to be
;         passed to the constructor. Otherwise (since loadSequence destroys all
;         existing dicom objects) the dicom matrix is created automatically
;         from information in the dicom file - slice position and/or time.
; Author: (c) Oliver Thilmann, oliver.thilmann@radfys.lu.se
; Licns : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------

;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::init
; Syntax: obj = obj_new('mrImageSetDicom', d0, d1, d2, name=name, class=class )
; Args  : <b>d0, d1, d2</b>
;         Dimensions of the image pointer matrix - optional!
;         <b>name, class</b>
;         Two optional strings to set the class attributes 'name' and 'class'.
; Desc  : Instanciates an object of class mrImageSetDicom
;         If d0/d1/d2 are not provided, they are set after loading a sequence or
;         time sequence.
;=-----------------------------------------------------------------------------
function mrImageSetDicom::init, d1, d2, d3, name=name, class=class

		; check arguments. d1,2,3 are set to '0' as default - the constructor
		; of mrImageSet handles that correctly as non-existing dimensions

	ndims = 0
	d1 = ( n_elements(d1) eq 0 ) ? 0 : d1
	d2 = ( n_elements(d2) eq 0 ) ? 0 : d2
	d3 = ( n_elements(d3) eq 0 ) ? 0 : d3
	if( d1 ne 0 ) then ndims=ndims+1
	if( d2 ne 0 ) then ndims=ndims+1
	if( d3 ne 0 ) then ndims=ndims+1
	name     = n_elements( name    ) eq 0 ? '' : name
	class    = n_elements( class   ) eq 0 ? '' : class

		; call mrImageSet constructor

	res = self->mrImageSet::init( d1, d2, d3, name=name, class=class )
	if( res eq 0 ) then return, 0

		; create empty pointer matrix for mrDicom objects

	if( self.ndims gt 0 ) then begin
		dicomdata = objarr( self.dims[0:self.ndims-1] )
		self.dicomdata = ptr_new( dicomdata, /no_copy )
	end else begin
		self.dicomdata = ptr_new()
	end
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::cleanup
; Syntax: obj_destroy, obj
; Desc  : The destructor - deletes all data by calling mrImageSetDicom::unload
;=-----------------------------------------------------------------------------
pro mrImageSetDicom::cleanup
	self->delete, /all
	ptr_free, self.dicomdata
end


;==============================================================================
;
;
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::seriesNumber
; Syntax: data = obj->dicomData()
; Ret   :
;=-----------------------------------------------------------------------------
function mrImageSetDicom::seriesNumber
	return, self.seriesNumber
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::studyUID
; Syntax: data = obj->dicomData()
; Ret   :
;=-----------------------------------------------------------------------------
function mrImageSetDicom::studyUID
	return, self.studyUID
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::mrImageSet
; Syntax: data = obj->mrImageSet
; Ret   : Returns a newly created mrImageSet object which is a copy of the
;         underlying mrImageSet object to this mrImageSetDicom object.
;         The original reason to introduce this method was related to
;         the condense method.
;         Maybe this method should go.
;=-----------------------------------------------------------------------------
function mrImageSetDicom::mrImageSet
	self.palette->getProperty, blue_values=b, green_value=g, red_value=r
	case self.ndims of
		0: is = obj_new('mrImageSet', 			 0, blue=b, green=g, red=r, xsize=self.xsize, ysize=self.ysize )
		1: is = obj_new('mrImageSet', self.dims[0], blue=b, green=g, red=r, xsize=self.xsize, ysize=self.ysize )
		2: is = obj_new('mrImageSet', self.dims[0], self.dims[1], blue=b, green=g, red=r, xsize=self.xsize, ysize=self.ysize )
		3: is = obj_new('mrImageSet', self.dims[0], self.dims[1], self.dims[2], blue=b, green=g, red=r, xsize=self.xsize, ysize=self.ysize )
	end
	is->setName,  self.name+' copy'
	is->setClass, self.class+' copy'
	pointermatrix = *self.imgdata
	nimgs = size(pointermatrix, /n_elements)
	for i = 0, nimgs-1 do begin
		pointermatrix[i] = ptr_new( *pointermatrix[i] )   ; creates a copy
	end
	is->setMatrix, ptr_new( pointermatrix, /no_copy )
	return, is
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::dicomData
; Syntax: data = obj->dicomData()
; Ret   : Returns a pointer to the matrix of pointers to mrDicom objects
;=-----------------------------------------------------------------------------
function mrImageSetDicom::dicomData
	return, self.dicomData
end

;------------------------------------------------------------------------------
; Name  : mrImageSetDicom::save
; Desc  : not implemented yet - use mrdicom::write for writing of dicom files
;------------------------------------------------------------------------------
function mrImageSetDicom::saveDicom
	message, 'Not implemented'
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::getTagValue
; Syntax: res = mrImageSetDicom::getTagValue( dicom, group, element )
; Desc  : Private method!
;         Given group and element id, the function returns the dicom value that
;         does not belong to a sequence (i.e. the one whose parent id is -1)
;         We do that here instead of mrDicom since the filling of mrDicom
;         internal structures takes quite some time - too long time for the
;         loading of ~ 1k images.
;=-----------------------------------------------------------------------------
;         The old comment:
;         Unluckily the Philips images contain several sequences/items, each containing
;         an instance number, so we can't just use pval[0], but need to find the
;         sequence with no parent
;         See Gary Muswick's answer in comp.lang.idl-pvwave 'IDLffDicom and multiple items'
function mrImageSetDicom::getTagValue, dicom, group, element
	vals    = dicom->getValue( group, element, /no_copy )
	refs    = dicom->getReference( group, element )
	parents = dicom->getParent( refs )
	id = where( parents eq -1 )
	return, vals[id[0]]
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::load
; Syntax: res = obj->load( filename, index, /rotate )
; Args  : <b>filename</b>
;         The file to load, full path
;         <b>index</b>
;         An n-element array (where n is the number of matrix dimensions)
;         holding the position where to store the loaded image internally.
;         <b>rotate</b>
;         If set, the image is rotated by 180 degrees and sides flipped
;         (via rotate( image, 7) )
; Ret   : 0 in case of error, 1 in case of success.
; Desc  : Load a single image.
; Code  : set = obj_new('mrImageSetDicom', 10, 60)
;         ok  = set->load( filename, [9,37])
;         if( ok ) then print, 'Loading succeeded' else print, 'Fail'
;=-----------------------------------------------------------------------------
function mrImageSetDicom::load, filename, index, rotate=rotate
	if( n_elements(filename) eq 0       ) then return, 0
	if( n_elements(index) ne self.ndims ) then return, 0
	rot = keyword_set( rotate )
	return, self->loadImgs( 'single', index, filename, rotate=rot )
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::loadSequence
; Syntax: res = obj->loadSequence( filename, filter=filter, /rotate, /progressbar )
; Args  : <b>filename</b>
;         One of the files in the set to load
;         <b>filter</b>
;         The method loads a complete image series, i.e. images that have the
;         same series number, into memory, where the series number is taken from
;         the initial file.
;         To do this, the method needs to load each single file in the directory
;         to check whether the series numbers match - a huge overhead, if many
;         files do not belong to the desired series.
;         If the filter keyword is set, only files are loaded and checked which
;         match the filter criteria. If it is possible to narrow down the selection
;         via some filter, loading might speed up considerably.
;         <b>rotate</b>
;         If set, rotate the loaded images via rotate( image, 7)
;         <b>progressbar</b>
;         If set, a progressbar is displayed while loading
; Ret   : 1 in case of success, else 0.
; Desc  : Loads a sequence of images (i.e. images that have the same series number)
;         and stores them in an array.
;         The pictures in the data array as well as the raw data (dicom array)
;         are ordered by image number.
; Bugs  : The method returns with an error if only one image is loaded, requiring
;         sequences to consist of more than one single image. It should return
;         without an error message.
;=-----------------------------------------------------------------------------
function mrImageSetDicom::loadSequence, filename, progressbar=progressbar, filter=filter, rotate=rotate
    if( n_elements(filename) eq 0 ) then return, 0
    pb  = ( n_elements(progressbar) eq 0 ) ? 0 : progressbar
    fl  = ( n_elements(filter)      eq 0 ) ? '*' : filter
    rot = keyword_set( rotate )
	return, self->loadImgs( 'sequence', [0], filename, filter=fl, progressbar=pb, rotate=rot )
end

;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::loadTimeSequence
; Syntax: res = obj->loadTimeSequence( filename, /progressbar )
; Args  : <b>filename</b>
;         One of the files in the set to load
;         <b>filter</b>
;         See the comment in mrImageSetDicom::loadSequence
;         <b>rotate</b>
;         If set, rotate the loaded images via rotate( image, 7)
;         <b>progressbar</b>
;         If set, a progressbar is displayed while loading
; Ret   : 1 in case of success, else 0 - check mrImageSet::errMsg for an
;         error description in that case.
; Desc  : As mrImageSetDicom::loadSequence, but loads a time sequence of images and
;         stores them in a two dimensional matrix. The first matrix index denotes the
;         slice number, the second one the shot number (i.e. position in time).
;         The decision is based on the 3rd element (z-position) of the patient
;         image position (dicom tag (0020,0032)), which obviously requires different
;         z-positions for the slices. I.e. if the slices should be aligned parallel
;         to the z-axis, the sorting algorithm will not work.
; Bugs  : Attempts to load a 'normal' set of images via loadTimeSequence will
;         result in a (caught) IDL error message. It should be a 1xn matrix.
;=-----------------------------------------------------------------------------
function mrImageSetDicom::loadTimeSequence, filename, progressbar=progressbar, filter=filter, rotate=rotate
    if( n_elements(filename) eq 0 ) then return, 0
    pb  = ( n_elements(progressbar) eq 0 ) ? 0 : progressbar
    fl  = ( n_elements(filter)      eq 0 ) ? '*' : filter
    rot = keyword_set( rotate )
	return, self->loadImgs( 'timesequence', [0], filename, filter=fl, progressbar=pb, rotate=rot )
end


;==============================================================================
;
;                             PRIVATE OBJECT METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::loadImgs
; Syntax: res = obj->loadImgs( mode, index, filename, filter=filter, /progressbar, /rotate)
;=-----------------------------------------------------------------------------
function mrImageSetDicom::loadImgs, mode, index, filename, filter=filter, progressbar=progressbar, rotate=rotate

	i18n = mrI18n()

		; error handling

	error = 0
	catch, error
	if( error ne 0 ) then begin
		catch, /cancel
		print, !ERROR_STATE.msg
		self.errmsg = !ERROR_STATE.msg
		if( obj_valid(pb) ) then obj_destroy, pb
		return, 0
	end
	self.errmsg = ''

	rot = keyword_set( rotate )  ; rotate images?

		; check for argument validity and set some defaults

	if( n_elements(filename) eq 0 ) then message, i18n->get('isd_missingFilename')
	pb     = ( n_elements(progressbar) eq 0 ) ?   0 : progressbar
	filter = ( n_elements(filter)      eq 0 ) ? '*' : filter

		; open file, get image properties, destroy the object again

	img = obj_new('mrDicom')
	res = img->read( filename )       ; res = 0, if not DICOM format
	if( res eq 0 ) then message, i18n->get('isd_errOpenFile') + ': ' + filename

		; all dicom data access is *not* done via getTagValue as this would
		; fill the internal mrdicom structure (which is slooooow)

	valptr = self->getTagValue( img, 32, 17 )                      ; x0020, x0011
	if( not ptr_valid(valptr) ) then begin
		obj_destroy, img
		message, i18n->get('isd_noSeriesNumber') + file
	end
	self.seriesnumber = *valptr

	valptr = self->getTagValue( img, 40, 16 )                      ; x0028, x0010   # of image rows
	if( not ptr_valid(valptr) ) then begin
		obj_destroy, img
		message, i18n->get('isd_missingImgRows')
	end
	self.ysize = *valptr

	valptr = self->getTagValue( img, 40, 17 )                      ; x0028, x0011   # of image columns
	if( not ptr_valid(valptr) ) then begin
		obj_destroy, img
		message, i18n->get('isd_missingImgCols')
	end
	self.xsize = *valptr
	self.studyUID = img->getTagValue('rel_study_instance_uid')     ; fast enough for just one

		;
		; SINGLE
		;

	if( mode eq 'single' ) then begin
		ptr = self->getTagValue( img, 32736, 16 )               ; x7fe0, x0010   the image data
		if( not ptr_valid(ptr) ) then begin                     ; no image data found
			obj_destroy, img
			message, i18n->get('isd_missingImgData') + filename
		endif
		id = index[self.ndims-1]
		for i = self.ndims-2, 0, -1 do id = index[i] + self.dims[i]*id
		(*self.imgdata)[id]      = ptr
		if( rot ) then *(*self.imgdata)[id] = rotate( *(*self.imgdata)[id], 7 )
		(*self.dicomdata)[id] = img
		self.path = dirname( filename )
		return, 1
	end

	obj_destroy, img

		;. Scan all files in the directory, load them, get the series number
		;. and store them in our list if the series number equals self.seriesnumber.
		;. A failed load will not cause a routine break but continue with the next file.
		;. If the failed load was crucial, this will lead to an error message further
		;. down when doing the validity checks

	self.path = dirname( filename )
	allfiles = findfile( self.path+'\'+filter, count=nfiles )
	if( nfiles eq 0 ) then return, 0                            ; should not happen, if filter ok

	io = objarr( nfiles )                                       ; mrdicom image objects
	no = 0                                                      ; # of loaded images
	if( progressbar ) then begin
		pb = obj_new( 'mrprogressbar', title=i18n->get('isd_loadingImages') )
	end
	for i = 0, nfiles-1 do begin                                ; no more than 2^16 pics, ok?
		if( progressbar ) then pb->update, float(i)/(n_elements(allfiles)-1)
		io[no] = obj_new( 'mrDicom', allfiles[i] )              ; load the file
		if( io[no] eq obj_new() ) then continue                 ; if load failed -> next
		valptr = self->getTagValue( io[no], 32, 17 )            ; get series number
		if( not ptr_valid(valptr) ) then begin                  ; no number found?
			obj_destroy, io[no]									; -> destroy object and cont
			continue
		end
		if( *valptr ne self.seriesnumber ) then begin           ; wrong series number?
			obj_destroy, io[no]                                 ; -> destroy object
			continue                                            ;    and continue with next
		end
		no=no+1                                                 ; otherwise we got a valid image
	end
	if( progressbar ) then obj_destroy, pb

		; if filter is wrong, no images may have been found

	if( no lt 2 ) then begin
		for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
		if( no eq 0 ) then message, i18n->get('isd_noValidImgsFound')
		if( no eq 1 ) then message, i18n->get('isd_onlyOneImgFound')
	end

		;. check whether all image numbers are present and sort images

	imgid = intarr( no )                                       ; array for image number
	err   = 0;
	for i = 0, no-1 do begin
		pval = self->getTagValue( io[i], 32, 19 )              ; x20, x13: image number
		if( not ptr_valid(pval) ) then begin
			err=err+1
			continue
		end
		imgid[i] = *pval
	end

	if( err gt 0 ) then begin                                  ; if we encountered errors
		for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
		message, i18n->get('isd_missingImgNumber')
	end

	order  = sort(imgid)                                       ; get images in consecutive order
	imgid  = imgid[order]                                      ; sort images
	io     = io[order]                                         ; sort raw data by image number
	firstid = imgid[0]
	for i = 0, no-1 do begin
		if( imgid[i] ne firstid+i ) then begin
			for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
			message, i18n->get('isd_imgNumbersNotConsecutive')  + $
			         i18n->get('isd_firstId') + string(firstid) + i18n->get('isd_firstMissingId') + string(firstid+i)
		end
	end

		;
		; SEQUENCE
		; this will destroy all currently stored images and mrDicom objects
		;

	if( mode eq 'sequence' ) then begin
		imgdata   = ptrarr( no )
		dicomdata = objarr( no )
		for i = 0, no-1 do begin                                      ; loop over all images
			ptr = self->getTagValue( io[i], 32736, 16 )               ; get image data
			if( not ptr_valid(ptr) ) then begin                       ; no image data found
				self->unload, progressbar=1
				message, i18n->get('isd_missingImgDataInImg') + string(i)
			endif
			imgdata[i]   = ptr
			if( rot ) then *imgdata[i] = rotate( *imgdata[i], 7 )
			dicomdata[i] = io[i]                                      ; io may contain many empty cells
		end
		self->delete, /all
		ptr_free, self.imgdata
		ptr_free, self.dicomdata
		self.ndims     = 1
		self.dims[0]   = no  ; number of images loaded
		sizes          = size( *imgdata[0], /dimensions)
		self.xsize     = sizes[0]
		self.ysize     = sizes[1]
		self.imgdata   = ptr_new( imgdata, /no_copy )
		self.dicomdata = ptr_new( dicomdata, /no_copy )
		return, 1                                                     ; EOF
	end

		;
		; TIME-SEQUENCE
		; this will destroy all currently stored images and mrDicom objects
		;

	if( progressbar ) then begin
		pb = obj_new( 'mrprogressbar', title=i18n->get('isd_scanningImgs') )
	end
	slices = dblarr( no )                                         ; arr for slice locations
	for i = 0, no-1 do begin
		if( progressbar ) then pb->update, float(i)/(n_elements(io)-1)
		pval = self->getTagValue( io[i], 32, 50 )                 ; get slice location string
		if( not ptr_valid(pval) ) then begin
			for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
			message, i18n->get('isd_missingSlicePosInfo')
		end
		pos       = strsplit( *pval, '\', /extract )
		slices[i] = double(pos[2])
	end
	if( progressbar ) then obj_destroy, pb

		;. get number of different slice positions

	nslices=n_elements(uniq(slices[sort(slices)]))               ; almost Perl :-)

		;. total number of images must be a multiple of nofslices

	if( no mod nslices ne 0 ) then begin
		for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
		message, i18n->get('isd_imgsBySlicesNotNull')
	end
	nshots  = no/nslices

		;. find out whether images are ordered in time or in space, then
		;. store the image data in an array

	imgdata   = ptrarr( nslices, nshots )
	dicomdata = objarr( nslices, nshots )

	if( slices[0] eq slices[1] ) then begin                       ; ordered in space
		for i = 0, no-1 do begin
			ptr = self->getTagValue( io[i], 32736, 16 )           ; get image data
			if( not ptr_valid(ptr) ) then begin
				for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
				message, i18n->get('isd_missingImgDataInImg') + string(i)
			end
			imgdata[i/nshots, i mod nshots]   = ptr
			if( rot ) then *imgdata[i/nshots, i mod nshots] = rotate( *imgdata[i/nshots, i mod nshots], 7 )
			dicomdata[i/nshots, i mod nshots] = io[i]
		end
	endif else begin                                              ; ordered in time
		for i = 0, no-1 do begin
			ptr = self->getTagValue( io[i], 32736, 16 )           ; get image data
			if( not ptr_valid(ptr) ) then begin                   ; no image data found
				for i = 0, no-1 do if( obj_valid(io[i]) ) then obj_destroy, io[i]
				message, i18n->get('isd_missingImgDataInImg') + string(i)
			endif
			imgdata[i mod nslices, i/nslices]   = ptr
			if( rot ) then *imgdata[i mod nslices, i/nslices] = rotate( *imgdata[i mod nslices, i/nslices], 7 )
			dicomdata[i mod nslices, i/nslices] = io[i]
		end
	end

		; LOADING was successful - transfer all data to object vars

	self->delete, /all
	ptr_free, self.imgdata
	ptr_free, self.dicomdata
	self.ndims     = 2
	self.dims[0:1] = [nslices, nshots]
	sizes          = size( *imgdata[0], /dimensions)
	self.xsize     = sizes[0]
	self.ysize     = sizes[1]
	self.imgdata   = ptr_new( imgdata, /no_copy )
	self.dicomdata = ptr_new( dicomdata, /no_copy )

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mrImageSetDicom::delete
; Syntax: obj->delete, slice, shot, /progressbar, /all
; Args  : <b>slice, shot</b>
;         Slice and shotnumber of the image to delete. Shot is '1' by
;         default.
;         <b>progressbar</b>
;         If set, a progressbar will be instanciated and update to show
;         deletion progress. Only appropriate for very large data sets or very
;         slow machines.
;         <b>all</b>
;         If set to '1', all images will be 'free'd. Arguments <i>slice</i>
;         and <i>shot</i> are ignored in this case (and need not be set).
; Desc  : Deletes one or all image(s) from the data set.
;=-----------------------------------------------------------------------------
pro mrImageSetDicom::delete, index, progressbar=progressbar, all=all

	i18n = mrI18n()

	all         = ( n_elements(all)         eq 0 ) ? 0 : all
	progressbar = ( n_elements(progressbar) eq 0 ) ? 0 : progressbar

		; we first take care of the dicom objects:

	if( all eq 1 ) then begin
		if( progressbar ) then begin
			pb = obj_new('mrprogressbar', title=i18n->get('isd_deletingDicomObjs') )
		end
		nimgs = self->nimgs()
		for i = 0, nimgs-1 do begin
			if( obj_valid( (*self.dicomdata)[i]) ) then obj_destroy, (*self.dicomdata)[i]
			(*self.dicomdata)[i] = obj_new()
			if( progressbar ) then pb->update, (1.+i)/nimgs
		end
		if( obj_valid(pb) ) then obj_destroy, pb
		self.xsize     = 0
		self.ysize     = 0
	end else begin
		if( n_elements(index) ne self.ndims ) then return
		id = index[self.ndims-1]
		for i = self.ndims-2, 0, -1 do id = index[i] + self.dims[i]*id
		if( obj_valid((*self.dicomdata)[id]) ) then obj_destroy, (*self.dicomdata)[id]
		(*self.dicomdata)[id] = obj_new()
	end
	self.errmsg    = ''

		; the superclass takes care of the images and the notification of registered
		; objects

	self->mrImageSet::delete, index, progressbar=progressbar, all=all
end


;+=----------------------------------------------------------------------------
; Name : mrImageSetDicom__define
; Desc : The constructor.
;=-----------------------------------------------------------------------------
pro mrImageSetDicom__define
	obj = { MRIMAGESETDICOM         			,$
			inherits MRIMAGESET     			,$
			dicomdata:				ptr_new()	,$; ptr to a matrix of mrdicom references
			seriesnumber:                   0   ,$; the series number of this set
			studyUID:                      ''    $; the study UID of this set
	}
end
