;+=----------------------------------------------------------------------------
; Name   : mrpan
; RCS    : $Id: mrpan__define.pro,v 2.1 2004/11/10 10:58:17 olivert Exp olivert $
; Methods: <b>Constructor</b>
;          mrPan::init                : constructor, part II
;          <br>
;          <b>Event handling</b>
;          mrPan::selfregister        : register with a display object
;          mrPan::unregister          : unregister
;          <br>
;          <b>Private</b>
;          mrPan::mrDisplayEvtHandler : the event handler and broker for mr*display
;          mrPan::buttonDblClickEvt   : handling of double clicks
;          mrPan::buttonPressEvt      : handling of button presses
;          mrPan::buttonReleaseEvt    : handling of button release
;          mrPan::buttonMotionEvt     : handling of motion while button pressed
;          mrPan__define              : the constructor
;          mrPan::cleanup             : the destructor
; Desc  :  A plugin object for mrdisplay which provides scrolling (i.e. movement
;          of the image) functionality. See mrzoom, which works similar, for a
;          code example.
;          To speed up things on slow computers, interpolation is switched off
;          while scrolling the image.
; Author:  (c) Oliver Thilmann, oliver.thilmann@web.de
; Licns :  This program is published under the LGPL. See gpl.txt for details
;=-----------------------------------------------------------------------------


;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrPan::init
; Syntax: x = obj_new('mrPan')
; Desc  : The init function. Does nothing in this case.
;=-----------------------------------------------------------------------------
function mrpan::init
	self.noborder = 0
	self.active   = 0
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mrPan::cleanup
; Type  : Private procedure
; Desc  : unregister from the display object during cleanup. For this reason,
;         it is a good idea to destroy the mrpan object before destroying
;         the associated mrdisplay object (or unregister explicitly)
;=-----------------------------------------------------------------------------
function mrpan::cleanup
	self->unregister
end


;==============================================================================
;
;                                  EVENT HANDLING
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrPan::selfregister
; Syntax: obj->selfregister, display, button=int, /noborder
; Args  : <b>display</b>
;         The mrdisplay object to register with.
;         <b>button</b>
;         The button number upon which to zoom; left=1 (def), middle=2, right=3
;         <b>noborder</b>
;         If set, mouse events in the right 15% of the mrDisplay object are
;         ignored. To be combined with the corresponding /border option
;         or mrZoom. When /noborder is set, button should be set to '2'
; Desc  : Register with an mrDisplay object
;=-----------------------------------------------------------------------------
pro mrpan::selfregister, display, button=button, noborder=noborder
	button = ( n_elements(button) eq 0 ) ? 1 : button
	if( button lt 1 ) then begin
		print, 'Illegal button number in mrpan::selfregister'
		return
	end
	self.noborder = keyword_set( noborder )
	display->registerEvt, self, ['Press', 'DblClick', 'Release','Motion']
	self.display = display
	self.button  = 2^(button-1)               ; internally left button = '0'
	return
end

;+=----------------------------------------------------------------------------
; Name : mrPan::unregister
; Desc : unregisters with a mrdisplay object. It is a good idea to unregister
;        before destroying the mrdisplay object.
;=-----------------------------------------------------------------------------
pro mrpan::unregister
	if( obj_valid( self.display )) then begin  ; mrdisplay object might be gone
		self.display->unregisterObj, self
	end
	self.display = obj_new()
end


;+=----------------------------------------------------------------------------
; Name : mrPan::mrDisplayEvtHandler
; Args : mrDisplayEvt *evt : pointer to the mrdisplay event structure
; Desc : Dispatch the event coming from mrdisplay
;=-----------------------------------------------------------------------------
pro mrPan::mrdisplayEvtHandler, evt
	if( ((*evt).button and self.button ) ne self.button ) then return
	case (*evt).type of
		0:	self->buttonPressEvt,    evt
		1:	self->buttonDblClickEvt, evt
		2:	self->buttonReleaseEvt,  evt
		3:	self->buttonMotionEvt,   evt
		else: return
	end
end

;+=----------------------------------------------------------------------------
; Name : mrPan::buttonDblClickEvt
; Args : mrdisplayEvt *evt : pointer to the mrdisplay event structure
; Desc : center image (independent of current scaling)
;=-----------------------------------------------------------------------------
pro mrpan::buttonDblClickEvt, evt
	if( self.noborder and (*evt).x gt 0.85 * self.display->screenXsize() ) then return
	self.display->centerImg
	self.display->requestDisplay
end

;+=----------------------------------------------------------------------------
; Name : mrPan::buttonPressEvt
; Args : mrdisplayEvt *evt : pointer to the mrdisplay event structure
; Desc : switch possible interpolation of while scrolling
;=-----------------------------------------------------------------------------
pro mrpan::buttonPressEvt, evt
	if( self.noborder and (*evt).x gt 0.85 * self.display->screenXsize() ) then return
	self.active=1
	self.interpolate = self.display->interpolate()
	if( self.interpolate eq 0 ) then return
	self.display->setInterpolate, 0
	self.display->requestDisplay
end

;+=----------------------------------------------------------------------------
; Name : mrPan::buttonReleaseEvt
; Args : mrdisplayEvt *evt : pointer to the mrdisplay event structure
; Desc : if interpolation was switched of during scrolling, now switch it
;        on again
;=-----------------------------------------------------------------------------
pro mrpan::buttonReleaseEvt, evt
	self.active=0
	if( self.interpolate eq 0 ) then return
	self.display->setInterpolate, self.interpolate
	self.display->requestDisplay
end

;+=----------------------------------------------------------------------------
; Name : mrPan::buttonMotionEvt
; Args : mrDisplayEvt *evt : pointer to the mrdisplay event structure
; Desc : move the image position in display
;=-----------------------------------------------------------------------------
pro mrpan::buttonMotionEvt, evt
	if( self.active eq 0 ) then return
	self.display->scroll, (*evt).dx, (*evt).dy
	self.display->requestDisplay
end

;==============================================================================
;
;                             PRIVATE OBJECT METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrPan__define
; Type  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mrpan__define
	obj = { MRPAN, $
			display		:	obj_new()	,$
			button		:	0			,$
			noborder	:   0			,$ ; if set, no panning with mouse near right border
			active		:	0			,$ ; flag needed for border operation
			interpolate	:	0			 $ ; temporary storage for the display's interp.
		}
end
