;+=----------------------------------------------------------------------------
; Name   : mrparameter
; RCS    : $Id: mrparameter__define.pro,v 2.0 2004/08/05 18:10:00 olivert Exp olivert $
; Methods:
;          <b>Constructor</b>
;          mrparameter::init           : constructor, 2nd part
;          <br>
;          <b>Public variables (get)</b>
;          mrparameter::name           : returns the name of this parameter
;          mrparameter::type           : returns its type, 'int', 'real' or 'string'
;          mrparameter::limits         : return lower/upper limits and step size
;          mrparameter::validValues    : return them (as array)
;          mrparameter::size           : returns the the number of elements
;          <br>
;          <b>Public variables (set)</b>
;          mrparameter::set            : set the parameter value(s)
;          mrparameter::get            : return the parameter value(s)
;          mrparameter::setLimits      : set lower/upper limits and step size
;          mrparameter::setValidValues : for discrete or text parameters
;          mrparameter::setSize        : sets the number of elements
;          mrparameter::check          : check validity of current value(s)
;          <br>
;          <b>Private methods</b>
;          mrparameter::dump           : dump object to screen (for debugging)
;          mrparameter__writedtd       : static method, writes part of DTD
;          mrparameter::write          : write object content to file in XML format
;          mrparameter__define         : constructor, 1st part
;          mrparameter::cleanup        : the destructor
; Desc   : An object to hold a numerical or character parameter with 1 or
;          more elements identified via a name.
;          mrparameter is meant to be used with mrparameterset which acts
;          as a container for a group of parameters belonging together.
; Author: (c) Oliver Thilmann, oliver.thilmann@web.de
; Licns : This program is published under the LGPL. See gpl.txt for details
;=-----------------------------------------------------------------------------


;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrparameter::init
; Syntax: obj = obj_new('mrparameter', name, type, size)
; Args  : <b>name</b>
;         The name of this parameter. Parameter names within a parameterset
;         serve as a key, and thus have to be unique.
;         <b>type</b>
;         String denoting the type of the parameter. One of 'int', 'real' or
;         'string', default is 'real'.
;         <b>size</b>
;         The number of values stored in this parameter. May be changed
;         dynamically explicitly (via the setSize method) or implicitly by
;         adding a large value set. Default size is '1'.
; Desc  : Create a new mrparameter object with name 'name'.
;         Once a parameter object is created, it cannot change name nor type.
;=-----------------------------------------------------------------------------
function mrparameter::init, name, type, size

	if( not keyword_set(name) ) then return, 0
	if( name eq ''          )   then return, 0

	type      = (n_elements(type) eq 0 ) ? 'real' : type
	size      = (n_elements(size) eq 0 ) ?      1 : size
	case type of
		'int'   : begin
			self.type = 0
			arr = intarr( size )
		end
		'real'  : begin
			self.type = 1
			arr = dblarr( size )
		end
		'string': begin
			self.type = 2
			arr = strarr( size )
		end
		else    : return, 0
	end
	self.name      = name
	self.limits[0] = 0.                        ; no limits set
	self.values    = ptr_new( arr, /no_copy )
	self.valid     = ptr_new()
	self.size      = size
	self.nvalid    = 0               ; nothing stored
	self.svalid    = 0               ; no array allocated yet
	return, 1
end

;+=----------------------------------------------------------------------------
; Name : mrparameter::cleanup
; Desc : The destructor
;=-----------------------------------------------------------------------------
pro mrparameter::cleanup
	ptr_free, self.values
	ptr_free, self.valid
end

;==============================================================================
;
;                              PUBLIC CLASS METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrparameter::set
; Syntax: obj->set, value, position=pos, /delete
; Args  : <b>value</b>
;         Single value or value array of the correct type
;         <b>position</b>
;         An integer denoting the position at which to set (or start, if
;         'value' is an array) the variables.
;         <b>delete</b>
;         Delete currently set values, before possibly setting new ones.
;         The method may be used with only the /delete keyword set, to
;         clear all values. This will of course not change the size of the
;         parameter.
; Desc  : Sets mrparameter value or values. If the parameter holds more than
;         a single value, the value array or a subarray can be set at once,
;         starting at position 'position'.
;         If there's more values than would fit in the current parameter set,
;         the size is automatically enlarged to fit the extra values.
;         There's no type checking - the method will crash if string values are
;         assigned to a parameter of numeral type.
;=-----------------------------------------------------------------------------
pro mrparameter::set, value, position=pos, delete=delete

	delete = ( n_elements(delete) eq 0 ) ? 0 : delete
	pos    = ( n_elements(pos) eq 0    ) ? 0 : pos

	if( delete eq 1 ) then begin
		ptr_free, self.values
		self.size = self.size > (n_elements(value)+pos)
		case self.type of
			0: arr = intarr(self.size)
			1: arr = dblarr(self.size)
			2: arr = strarr(self.size)
		end
		self.values = ptr_new( arr, /no_copy )
	end
	if( n_elements(value) eq 0 ) then return      ; nothing to do

	lastid = pos + n_elements(value)-1            ; need more space?
	if( lastid ge self.size ) then begin          ; reallocation time....
		case self.type of
			0: newarr = intarr(lastid+1)
			1: newarr = dblarr(lastid+1)
			2: newarr = strarr(lastid+1)
		end
		newarr[0:self.size-1] = *self.values
		ptr_free, self.values
		self.values = ptr_new( newarr, /no_copy )
		self.size   = lastid+1
	end

		; now we have in any case enough space to copy the data

	(*self.values)[pos:pos+n_elements(value)-1] = value[*]
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::get
; Syntax: result = obj->get( position=pos )
; Args  : <b>position</b>
;         Element position whose value to return
; Desc  : Return the element at position 'position'.
;         If pos is not specified, all values (i.e. a single value or an array)
;         will be returned.
;         If position points at an invalid position, a 'message' is thrown.
;=-----------------------------------------------------------------------------
function mrparameter::get, position=pos

	if( n_elements(pos) eq 0 ) then $            ; pos not specified -> return all
		if( self.size eq 1 ) then $
			return, (*self.values)[0] else return, (*self.values)[*]

	if( n_elements(pos) gt 1 ) then message, "Position may only have 1 element"
	if( pos[0] lt 0 or pos[0] ge self.size ) then message, "Position out of bounds"

	return, (*self.values)[pos]                   ; return the value
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::name
; Syntax: result = obj->name()
; Desc  : Return the parameter's name
;=-----------------------------------------------------------------------------
function mrparameter::name
	return, self.name
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::size
; Syntax: result = obj->size()
; Desc  : Return the number of elements of the parameter
;=-----------------------------------------------------------------------------
function mrparameter::size
	return, self.size
end


;+=----------------------------------------------------------------------------
; Name  : mrparameter::setSize
; Syntax: obj->setSize, size
; Args  : <b>size</b>
;         Integer, denoting the number of elements of the parameter.
; Desc  : Sets the number of elements for the parameter. This might enlarge
;         or shrink the current parameter - accordingly parameter values might
;         get lost. When enlarging, the extra parameters are set to 0, 0. or ''
;         respectively.
;=-----------------------------------------------------------------------------
pro mrparameter::setSize, size
	if( n_elements(size) eq 0 ) then return
	if( size le 0 ) then return
	case self.type of
		0: newarr = intarr(size)
		1: newarr = dblarr(size)
		2: newarr = strarr(size)
	end
	newarr[0:self.size-1] = (*self.values)[*]
	ptr_free, self.values
	self.values = ptr_new( newarr, /no_copy )
	return
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::type
; Syntax: result = obj->type()
; Desc  : Return parameter type as string, i.e. one of 'int', 'real' or 'string'
;=-----------------------------------------------------------------------------
function mrparameter::type
	case self.type of
		0: return, 'int'
		1: return, 'real'
		2: return, 'string'
	end
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::setLimits
; Syntax: obj->setLimits, low=low, up=up, step=step, error=err
; Args  : <b>low</b>
;         Integer or real (depending on the type) specifying the lower limit
;         <b>up</b>
;         Integer or real specifying the upper limit
;         <b>step</b>
;         Integer or real specifying the step size (meant for UIs)
;         <b>error</b>
;         Setting new limits might render currently set values out of bounds.
;         If the error keyword is set, the method will call mrparameter::check
;         and set the assigned variable to '1', in case the check failed.
;         See also mrparameter::check
; Desc  : Set the mentioned values - parameter objects of type 'string' don't
;         support this method and return immediately.
;=-----------------------------------------------------------------------------
pro mrparameter::setLimits, low=low, up=up, step=step, error=err
	if( self.type eq 2 ) then return     ; limits are not supported for strings
	if( n_elements(low) ne 0 ) then begin
		self.limits[0] = low
		self.vlimit[0] = 1
	end
	if( n_elements(up) ne 0 ) then begin
		self.limits[1] = up
		self.vlimit[1] = 1
	end
	if( n_elements(step) ne 0 ) then begin
		self.limits[2] = step
		self.vlimit[2] = 1
	end
	if( keyword_set(err) ) then err = self->check()
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::limits
; Syntax: obj->limits, low=low, up=up, step=step, error=err
; Args  : <b>low</b>
;         If specified, the assigned variable is set to the lower limit
;         <b>up</b>
;         If specified, the assigned variable is set to the upper limit
;         <b>step</b>
;         If specified, the assigned variable is set to the step size
;         <b>error</b>
;         If set, the assigned variable upon return holds a bitmap of undefined,
;         but requested values
; Desc  : Return limits and step size.
;         Request for undefined limit values (i.e. values were not set via
;         setLimits before) are ignored. The corresponding bit in 'err' is
;         set to '1' where bit1 = low, bit2 = up, bit3 = step.
;         Parameter objects of type 'string' do not support this method and
;         return immediately. Error is set to '-1' in that case.
;=-----------------------------------------------------------------------------
pro mrparameter::limits, low=low, up=up, step=step, error=err
	err=-1
	if( self.type eq 2 )  then return               ; no limits for strings
	err=0
	if( keyword_set(low) ) then   $
		if( self.vlimit[0] eq 1 ) then low  = self.limits[0] else err=err+1
	if( keyword_set(up) ) then    $
		if( self.vlimit[1] eq 1 ) then up   = self.limits[1] else err=err+2
	if( keyword_set(step) ) then  $
		if( self.vlimit[2] eq 1 ) then step = self.limits[2] else err=err+4
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::setValidValues
; Syntax: obj->setValidValues, values, /delete, error=err
; Args  : <b>values</b>
;         Single value or array of values of the correct type.
;         [Values] might be omitted - e.g. when deleting currently set
;         valid values a la <i>obj->setValidValues, /delete</i>
;         <b>delete</b>
;         If set to '1', the current list of valid values will be deleted
;         before adding the new ones. Default is to append to the list.
;         Using only delete without setting any values may be used to
;         clear the list of valid values.
;         <b>error</b>
;         Setting a valid value list may render currently set parameter values
;         invalid. If the error keyword is set, the method will call
;         mrparameter::check and set the assigned variable to '1', if the check
;         failed.
; Desc  : Define or append to a list of valid values the parameter may have.
;         Values are character or numerical constants depending on the parameter
;         type. There's no check for uniqueness (yet).
;=-----------------------------------------------------------------------------
pro mrparameter::setValidValues, values, delete=delete, error=err

	delete = (n_elements(delete) eq 1) ? delete : 0    ; default is append

	if( delete eq 1 ) then begin
		ptr_free, self.valid
		self.valid   = ptr_new()
		self.nvalid  = 0
		self.svalid  = 0
		if( n_elements(err) ne 0 ) then err = self->check()
	end

	nel = n_elements(values)
	if( nel eq 0 ) then return

	if( self.valid eq ptr_new() ) then begin     ; allocate exactly the amount needed
		case self.type of
			0: arr = intarr(nel)
			1: arr = dblarr(nel)
			2: arr = strarr(nel)
		end
		arr[*]      = values[*]
		self.valid  = ptr_new( arr, /no_copy )
		self.svalid = nel
		self.nvalid = nel
		if( n_elements(err) ne 0 ) then err = self->check()
		return
	end

		; if array size is too small, allocate new - at least 10 elements

	if( self.svalid lt self.nvalid+nel ) then begin
		incr = self.nvalid+nel-self.svalid > 10
		case self.type of
			0: newarr = intarr( self.svalid + incr )
			1: newarr = dblarr( self.svalid + incr )
			2: newarr = strarr( self.svalid + incr )
		end
		newarr[0:self.nvalid-1] = (*self.valid)[0:self.nvalid-1]
		ptr_free, self.valid
		self.valid = ptr_new( newarr, /no_copy )
		self.svalid = self.svalid+incr
	end

		; now there's enough space to add the new

	(*self.valid)[self.nvalid:self.nvalid+nel-1] = values[*]
	self.nvalid = self.nvalid+nel
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::validValues
; Syntax: result = obj->getValidValues( count=count )
; Args  : <b>count</b>
;         If set, contains the number of valid elements in result upon return.
; Desc  : Returns a pointer to an array of valid values or ptr_new() if none set
;=-----------------------------------------------------------------------------
function mrparameter::validValues, count=count
	count = self.nvalid
	if( self.valid eq ptr_new() ) then return, self.valid
	return, (*self.valid)[0:self.nvalid-1]
end


;+=----------------------------------------------------------------------------
; Name  : mrparameter::check
; Syntax: result = obj->check()
; Desc  : Returns '1', (=TRUE) if check passed, else '0'.
;         If a list of validValues is available the method will check against
;         this list, otherwise it will check for upper and lower limits.
;=-----------------------------------------------------------------------------
function mrparameter::check

		; check for valid value list - brute force

	if( self.valid ne ptr_new() ) then begin
		for i = 0, self.size-1 do begin
			for j = 0, self.nvalid-1 do begin
				if( (*self.values)[i] eq (*self.valid)[j] ) then goto, Next
			end
			return, 0
			Next:								; there must be a nicer way
		end
		return, 1
	end else begin								; otherwise check for limits
		if( self.type eq 2 ) then return, 0		; no limit check for strings
		if( self.vlimit[0] eq 1 ) then begin
			for i = 0, self.size-1 do begin
				if( (*self.values)[i] lt self.limits[0] ) then return, 0
			end
		end
		if( self.vlimit[1] eq 1 ) then begin
			for i = 0, self.size-1 do begin
				if( (*self.values)[i] gt self.limits[1] ) then return, 0
			end
		end
		return, 1
	end
end


;==============================================================================
;
;                              PRIVATE CLASS METHODS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrparameter::dump
; Syntax: obj->dump
; Desc  : dump the contents of this parameter object - meant for debugging only
;=-----------------------------------------------------------------------------
pro mrparameter::dump
	print, '--- Parameter dump ---'
	print, '    Name  : ', self.name
	print, '    Type  : ', self.type
	print, '    Size  : ', self.size
	if( self.vlimit[0] ) then print, strtrim(string(self.limits[0]),2), format='("    Low   : ",a)'
	if( self.vlimit[1] ) then print, strtrim(string(self.limits[1]),2), format='("    Up    : ",a)'

	if( self.vlimit[2] ) then print, strtrim(string(self.limits[2]),2), format='("    Step  : ",a)'
	if( self.valid ne ptr_new() ) then begin
		print, '    Valid : '
		for i = 0, self.nvalid-1 do begin
			print, strtrim(string((*self.valid)[i]),2), format='("        ",a)'
		end
	end
	print, '    Values: '
	for i = 0, self.size-1 do begin
		print, strtrim(string((*self.values)[i]),2), format='("        ",a)'
	end
end


;+=----------------------------------------------------------------------------
; Name  : mrparameter__writedtd
; Syntax: obj->write, unit
; Args  : <b>unit</b>
;         Logical unit to write to. This is a 'static' method.
; Desc  : Write the part of the DTD that builds a single parameter
;=-----------------------------------------------------------------------------
pro mrparameter__writedtd, unit
	printf, unit, '<!ELEMENT Parameter (l?, u?, s?, o*, v*)>'
	printf, unit, '<!ATTLIST Parameter name CDATA #REQUIRED>'
	printf, unit, '<!ATTLIST Parameter type (int|real|string) #REQUIRED>'
	printf, unit, '<!ATTLIST Parameter size CDATA #REQUIRED>'
	printf, unit, '<!ELEMENT l (#PCDATA)>' ; low
	printf, unit, '<!ELEMENT u (#PCDATA)>' ; up
	printf, unit, '<!ELEMENT s (#PCDATA)>' ; step
	printf, unit, '<!ELEMENT o (#PCDATA)>' ; valid = ok
	printf, unit, '<!ELEMENT v (#PCDATA)>' ; value
end

;+=----------------------------------------------------------------------------
; Name  : mrparameter::write
; Syntax: obj->write, unit, /valuesonly
; Args  : <b>unit</b>
;         Logical unit to write to
;         <b>valuesonly</b>
;         If set, write only values, but no limits and valid value lists.
;         Default is to write all.
; Desc  : Dump the content of a parameter object to file.
;=-----------------------------------------------------------------------------
pro mrparameter::write, unit, valuesonly=vo

	vo = ( n_elements(vo) eq 0 ) ? 0 : vo     ; default is to write all

		; write the <Parameter ...> line

	printf, unit, '"'+self.name+'"', '"'+self->type()+'"', '"'+strtrim(string(self.size),2)+'"', $
					format='(" <Parameter name=",a," type=", a, " size=",a,">")'

		; write limits, step size and values

	if( vo ne 1 ) then begin
		if( self.type ne 2 ) then begin                 ; no limits for strings
			if( self.vlimit[0] ) then printf, unit, strtrim(string(self.limits[0]),2), format='("  <l>",a,"</l>")'
			if( self.vlimit[1] ) then printf, unit, strtrim(string(self.limits[1]),2), format='("  <u>",a,"</u>")'
			if( self.vlimit[2] ) then printf, unit, strtrim(string(self.limits[2]),2), format='("  <s>",a,"</s>")'
		end
		if( self.valid ne ptr_new() ) then begin
			n = n_elements(*self.valid)
			for i = 0, self.nvalid-1 do begin
				printf, unit, strtrim(string((*self.valid)[i]),2), format='("  <o>",a,"</o>")'
			end
		end
	end
	for i = 0, self.size-1 do begin
		printf, unit, strtrim(string((*self.values)[i]),2), format='("  <v>",a,"</v>")'
	end

	printf, unit, " </Parameter>"
end


;+=----------------------------------------------------------------------------
; Name  : mrparameter__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mrparameter__define
	obj = { MRPARAMETER, $
			name		:		   ''	,$ ; the parameter's name (unique key)
			type		:		    0	,$ ; int (0), real(1) or string(2)
			values		:	ptr_new()	,$ ; pointer to an array of respective type
			valid		:	ptr_new()	,$ ; pointer to a list of valid values
			size        :		    0	,$ ; the increment per block of data
			svalues     :           0   ,$ ; array size in which values are stored
			nvalid		:	        0   ,$ ; number of valid values in array
			svalid      :           0   ,$ ; size of the valid value array
			limits		:	dblarr(3)	,$ ; integers have real limits as well. min,max,step
			vlimit		:	intarr(3)	 $ ; '1', if the corresponding limit is set
		}
end