;+=----------------------------------------------------------------------------
; Name  : mrRoi
; RCS   : $Id: mrroi__define.pro,v 2.0 2004/08/05 18:13:04 olivert Exp olivert $
; Desc  : Subclass to IDLgrRoi and superclass to the four different roi
;         types. See the init method for details.
;         <b>Constructors and destructor</b>
;         mrRoi__define            : constructor 1st part
;         mrRoi::init              : constructor
;         mrRoi::cleanup           : destructor
;         <b>Public variables (get/set)</b>
;         mrRoi::id                : return the roi id (an integer number)
;         mrRoi::setId             : set the roi id (an int)
;         mrRoi::duplicate         : returns a copy of the present object
;         mrRoi::copy              : copies all properties
;         <b>Public methods</b>
;         mrRoi::mask              : depending on mode, return filled or framed mask
;         mrRoi::filledMask        : returns the roi mask in the specified
;         .                          dimensions with all roi points set
;         mrRoi::framedMask        : returns the roi mask with only border
;         .                          points set
;         <b>Private methods</b>
;         mrRoi::calcFilledMask    : calculates the filled mask and stores it
;         mrRoi::calcFramedMask    : calculates the framed mask and stores it
; Author: (c) Oliver Thilmann, oliver.thilmann@web.de
; Licns : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------


;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrRoi::init
; Type  : Function
; Args  : int x           : as in IDLgrROI::init
;         int y           : as in IDLgrROI::init
;         int[2] maskSize : maskSize as in IDLanROI::computeMask
; Ret   : true or false
; Desc  : initialize a new mrRoi object. mrRoi is a subclass to IDLgrROI
;         and a superclass to the four ROI objects MRroiDot, MRroiCirc, MRroiRect
;         and MRroiPoly. Using its 'own' class of mrRoi objects enables mrroi
;         to do a) some easier bookkeeping and b) implement individual
;         getFramedMask and getFilledMask methods.
;         The latter is the same for all rois, but the framed mask needed a
;         different implementation: while for all roi objects, the frame is
;         part of the roi area, for MRroiDot it must (quite naturally, otherwise
;         it wouldn't be a dot) be excluded.
;=-----------------------------------------------------------------------------
function mrRoi::init, x, y, maskSize=z, _EXTRA=e
	if( n_elements(x) eq 0 or n_elements(y) eq 0 ) then return, 0
	if( self->IDLgrROI::init(x, y, _EXTRA=e) ne 1 ) then return, 0
	if( n_elements(z) eq 2 ) then self.masksize = z else self.masksize=[100,100]
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::cleanup
; Type  : Private procedure
; Desc  : free the two masks and destroy label (if present)
;=-----------------------------------------------------------------------------
pro mrRoi::cleanup
	ptr_free, self.framedmask
	ptr_free, self.filledmask
end


;==============================================================================
;
;                               GET AND SET METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrRoi::id
; Syntax: id = obj->getId()
; Ret   : the roi's id
;=-----------------------------------------------------------------------------
function mrRoi::id
	return, self.id
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::setId
; Syntax: obj->setId, id
; Args  : <b>id</b>
;         The roi id
;=-----------------------------------------------------------------------------
pro mrRoi::setId, id
	self.id = id
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::setMaskSize
; Syntax: obj->setMaskSize, masksize
; Args  : <b>masksize</b>
;         A two element integer array defining the size of the mask in x and y.
;=-----------------------------------------------------------------------------
pro mrRoi::setMaskSize, masksize
	if( n_elements(masksize) ne 2 ) then return
	self.masksize = masksize
end

function mrRoi::maskSize
	return, self.masksize
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::mask
; Syntax: mask = obj->mask( mode )
; Args  : <b>mode</b>
;         If mode = 0, a mask outlining the roi will be returned, otherwise
;         the filled mask.
;=-----------------------------------------------------------------------------
function mrRoi::mask, mode
	if( mode eq 0 ) then return, self->framedMask()
	return, self->filledMask()
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::filledMask
; Type  : Function
; Ret   : returns the 2-dim 'filled' mask in the dimensions as specified
;         in mrRoi::init . Calculates the mask, if necessary
;=-----------------------------------------------------------------------------
function mrRoi::filledMask
	if( self.filledMask eq ptr_new() ) then self->calcFilledMask
	return, self.filledMask
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::framedMask
; Type  : Public function
; Ret   : returns a (2-dim) framed mask in dimensions as specified in ::init.
;         Note that for all roi types except for MRroiDot, the frame is part
;         of the roi. For MRroiDot, it is not.
;         (MRroiDot overloads the calcFilledMask and calcFramedMask methods)
;=-----------------------------------------------------------------------------
function mrRoi::framedMask
	if( self.framedMask eq ptr_new() ) then self->calcFramedMask
	return, self.framedMask
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::calcFilledMask
; Type  : Public function
; Desc  : calculates the 2-dim filled mask (dimensions as defined during
;         instantiation) and saves a private pointer to it.
;=-----------------------------------------------------------------------------
pro mrRoi::calcFilledMask
	mask = self->computeMask( dimensions=self.maskSize, initialize=0, mask_rule=2)
	mask = mask and 1
	ptr_free, self.filledMask
	self.filledMask = ptr_new( mask, /no_copy )
end

;+=----------------------------------------------------------------------------
; Name  : mrRoi::calcFramedMask
; Type  : Public function
; Desc  : calculates the 2-dim framed mask (dimensions as defined during
;         instantiation) and saves a private pointer to it.
;=-----------------------------------------------------------------------------
pro mrRoi::calcFramedMask
	mask = self->computeMask( dimensions=self.maskSize, initialize=0, mask_rule=0)
	mask = mask and 1
	ptr_free, self.framedMask
	self.framedMask = ptr_new( mask, /no_copy )
end


;+=----------------------------------------------------------------------------
; Name  : mrRoi::duplicate
; Syntax: newObj = obj->duplicate()
; Desc  : Creates an exact duplicate of the object
;=-----------------------------------------------------------------------------
function mrRoi::duplicate
	class = obj_class(self)      ; so it also works correctly for subclasses
	new = obj_new(class, 0, 0 )
	if( not obj_valid(new) ) then return, obj_new()
	new->copy, self
	return, new
end


;+=----------------------------------------------------------------------------
; Name  : mrRoi::copy
; Syntax: obj->copy, source
; Args  : <b>source</b>
;         An mrRoi object to copy from
; Desc  : Copies all properties of the source object, i.e. creates an exact
;         copy.
;=-----------------------------------------------------------------------------
pro mrRoi::copy, source

	if( not obj_valid(source) ) then return

		; copy IDLgrROI properties - this also copies IDLanROI props

	source->IDLgrROI::getProperty, all=props
		; is there a nicer way for this? set_property, _extra=props does not work
	self->setProperty, color               = props.color[0]
	self->setproperty, block_size          = props.block_size
	self->setProperty, clip_planes         = props.clip_planes
	self->setproperty, depth_test_disable  = props.depth_test_disable
	self->setproperty, depth_test_function = props.depth_test_function
	self->setproperty, depth_write_disable = props.depth_write_disable
	self->setproperty, double              = props.double
	self->setproperty, hide                = props.hide
	self->setproperty, interior            = props.interior
	self->setproperty, linestyle           = props.linestyle
	self->setproperty, name                = props.name
	self->setproperty, palette             = props.palette
	self->setproperty, style               = props.style
	self->setproperty, symbol              = props.symbol
	self->setproperty, thick               = props.thick
	self->setproperty, xcoord_conv         = props.xcoord_conv
	self->setproperty, ycoord_conv         = props.ycoord_conv
	self->setproperty, zcoord_conv         = props.zcoord_conv

		; copy ROI vertices

	source->getProperty, data=vertices
	self->setProperty, data=vertices

		; copy our own properties

	ptr_free, self.framedmask
	ptr_free, self.filledmask
	self.id         = source.id
	self.masksize   = source.masksize
	self.framedmask = ptr_new()
	self.filledmask = ptr_new()

end

;+=----------------------------------------------------------------------------
; Name  : mrRoi__define
; Type  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mrRoi__define
	obj = { MRROI, $
			inherits IDLgrROI			,$ ; should be enough to inherit from IDLanROI?
			id			:			0	,$ ; roi id (e.g. number for labels)
			masksize	:	intarr(2)	,$ ; size of the mask in x and y
			framedmask	:	ptr_new()	,$ ;
			filledmask	:	ptr_new()	 $ ;
		}
end


